package com.sxtvs.open.api.bid.service.impl;

import com.alibaba.fastjson.JSON;
import com.sxtvs.open.api.bid.dto.FileImportDTO;
import com.sxtvs.open.api.bid.entity.BidFiles;
import com.sxtvs.open.api.bid.mapper.BidFilesMapper;
import com.sxtvs.open.api.bid.service.IBidFilesService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import jakarta.annotation.Resource;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.net.URLDecoder;
import java.nio.charset.StandardCharsets;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 标书文件 服务实现类
 * </p>
 *
 * @author syj
 * @since 2024-04-23
 */
@Service
public class BidFilesServiceImpl extends ServiceImpl<BidFilesMapper, BidFiles> implements IBidFilesService {

    public List<BidFiles> byProject(Long id){
        return lambdaQuery().eq(BidFiles::getProjectId, id).list().stream().peek(data -> {
            data.setColor(data.getSimilarity() >= 60 ? "red" : "green");
        }).collect(Collectors.toList());
    }

    @Transactional
    public void create(FileImportDTO fileImportDTO){
        List<BidFiles> bidFiles = new ArrayList<>();
        BidFiles base = new BidFiles();

        base.setProjectId(fileImportDTO.getProjectId());
        String decode = URLDecoder.decode(fileImportDTO.getBaseFile(), StandardCharsets.UTF_8);
        String[] split = decode.split("/");
        base.setFname(split[split.length - 1]);
        base.setFile(fileImportDTO.getBaseFile());
        base.setFileType(1);

        base.setCreateTime(LocalDateTime.now());
        bidFiles.add(base);

        for(String file : fileImportDTO.getCompareFiles()){
            BidFiles bf = new BidFiles();
            bf.setProjectId(fileImportDTO.getProjectId());
            String decode1 = URLDecoder.decode(file, StandardCharsets.UTF_8);
            String[] split1 = decode1.split("/");
            bf.setFname(split1[split1.length - 1]);
            bf.setFile(file);
            bf.setFileType(2);
            bf.setCreateTime(LocalDateTime.now());
            bidFiles.add(bf);
        }
        saveBatch(bidFiles);

    }

    public void deleteByProjectId(Long id){
        lambdaUpdate().eq(BidFiles::getProjectId, id).remove();
    }
}
