package com.sxtvs.open.api.chat.service.impl;

import cn.hutool.core.util.IdUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.sxtvs.open.api.chat.dto.KimiRequest;
import com.sxtvs.open.api.chat.dto.KimiStreamResponse;
import com.sxtvs.open.api.chat.dto.Message;
import com.sxtvs.open.api.chat.entity.KimiChat;
import com.sxtvs.open.api.chat.mapper.KimiChatMapper;
import com.sxtvs.open.api.chat.service.IKimiChatService;
import com.sxtvs.open.api.review.service.impl.SSEService;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.apache.hc.client5.http.classic.methods.HttpPost;
import org.apache.hc.client5.http.entity.mime.ContentBody;
import org.apache.hc.client5.http.entity.mime.MultipartEntityBuilder;
import org.apache.hc.client5.http.impl.classic.CloseableHttpClient;
import org.apache.hc.client5.http.impl.classic.CloseableHttpResponse;
import org.apache.hc.client5.http.impl.classic.HttpClients;
import org.apache.hc.core5.http.ContentType;
import org.apache.hc.core5.http.ParseException;
import org.apache.hc.core5.http.io.entity.EntityUtils;
import org.apache.http.util.TextUtils;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.multipart.MultipartFile;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.nio.channels.Channels;
import java.nio.channels.Pipe;
import java.nio.file.Paths;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Random;
import java.util.stream.Collectors;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author syj
 * @since 2024-03-25
 */
@Service
@Slf4j
public class KimiChatServiceImpl extends ServiceImpl<KimiChatMapper, KimiChat> implements IKimiChatService {

    private final RestTemplate restTemplate = new RestTemplate();

    private final List<String> token = Arrays.asList(
            "sk-eKQV5KAqkQHINWZV9IBbsuefO21A6oTToY9pv9nyDeXPH0zu",
            "sk-wMy3vyR7mCQc5vvr4uFm6lkfhdxFJZtQxx54pKvaJcVIa3XQ",
            "sk-ftWQ35trs1omUIUuKzsdeb6idKK9p6PzK4C9U5YTkHkt0PtD",
            "sk-9VL5bvkhGhtM3GhBD80orZm75vdGLrRH8xRF3Takl8Q82oea",
            "sk-mu6zmA4ZYg3xtayd5TLq0gXCOZNk8eOAGpTd7pri4inpFOXs",
            "sk-22Rh0RPkx7Z1uCKERWNSq8dgUaYZdSVy9d3d2otDyecGJZny",
            "sk-VRMUyGAnxqaCfnTOhttLUvysVLRz1MdwrBOyV6kvv6YKfk0s",
            "sk-PGZwUBAWeODTrMQC67u2Ag6966Bag9ooq4AytvbnijQ2UpsV"
            );

    private final Random random = new Random();

    @Resource
    private SSEService sseService;

    public void chatCompletions(KimiRequest kimiRequest, Long groupId) {

        HttpClient client = HttpClient.newBuilder().build();

        URI uri = URI.create("https://api.moonshot.cn/v1/chat/completions"); // 替换为你的SSE服务器地址
        HttpRequest request = HttpRequest.newBuilder()
                .uri(uri)
                .header("Accept", "text/event-stream")
//                .header("Content-Type", "application/json")
                .header("Authorization", "Bearer " + token.get(random.nextInt(token.size())))
                .POST( HttpRequest.BodyPublishers.ofString(JSON.toJSONString(
                    kimiRequest
                )))
                .build();
        log.info("发送kimi数据 {}",  JSON.toJSONString(kimiRequest));
        // 发送请求并接收响应
        client.sendAsync(request, HttpResponse.BodyHandlers.ofInputStream())
                .thenApply( HttpResponse::body )
                .thenAccept( resp -> {
                    BufferedReader reader = new BufferedReader(new InputStreamReader(resp));
                    try {
                        String line = "";
                        while ((line = reader.readLine()) != null) {
                            log.info("流数据 -- {} ", line);
                            if (!TextUtils.isEmpty(line)) {
                                line = line.substring(6);
                                sseService.sendData( groupId, line );
                                if (!"[DONE]".equals(line) && !line.isEmpty()) {
                                    stream2db(line, groupId);
                                }
                            }
                        }
                    } catch (IOException e) {
                        log.error("io错误 {}", e.getMessage());
                    } finally {
                        // 服务器端主动关闭时，客户端手动关闭
                        try {
                            reader.close();
                            resp.close();

                        } catch (IOException e) {
                            log.error("关闭链接异常 {}", e.getMessage());
                        }
                    }

                }).join();
        log.info( "--结束--" );
        sseService.complate(groupId);
    }

    private void stream2db(String resp, Long groupId){

        KimiStreamResponse kimiResponse ;
        try{
            kimiResponse =  JSON.parseObject(resp, KimiStreamResponse.class);
        }catch (Exception e){
            log.error("解析失败 -- {}",resp);
            return;
        }

        if (kimiResponse == null || kimiResponse.getChoices() == null || kimiResponse.getChoices().isEmpty()){
            return;
        }
        KimiChat kimiChat = lambdaQuery().eq(KimiChat::getId, kimiResponse.getId()).one();
        if  (kimiChat == null) {
            kimiChat = new KimiChat();
            kimiChat.setId(kimiResponse.getId());
            kimiChat.setGroupId(groupId);
            kimiChat.setInd(kimiResponse.getChoices().get(0).getIndex());
            kimiChat.setRole("system");
            kimiChat.setContent(kimiResponse.getChoices().stream().map(x -> x.getDelta().getContent( )).reduce((a, b) -> a + b).get());
            kimiChat.setCreateTime(LocalDateTime.now());
            save(kimiChat);
        }else {
            List<String> collect = kimiResponse.getChoices().stream().map(x -> x.getDelta().getContent()).collect(Collectors.toList());
            if (collect.size() == 0){
                return;
            }
            StringBuilder s = new StringBuilder(kimiChat.getContent());
            for (String x : collect){
                if (x != null){
                    s.append(x);
                }
            }
            kimiChat.setContent(s.toString());
            kimiChat.setUpdateTime(LocalDateTime.now());
            updateById(kimiChat);
        }
    }

    public KimiChat create(KimiChat  kimiChat) {

        kimiChat.setId("user-" + IdUtil.simpleUUID());
        kimiChat.setRole("user");
        kimiChat.setCreateTime(LocalDateTime.now());
        save(kimiChat);

        KimiRequest kimiRequest = new KimiRequest();
        kimiRequest.setModel("moonshot-v1-8k");
        kimiRequest.setTemperature(0.3D);
        kimiRequest.setStream(true);
        kimiRequest.setMax_tokens(4096);

        List<Message> messageList = new ArrayList<>();

        lambdaQuery().eq( KimiChat::getGroupId, kimiChat.getGroupId() ).orderByAsc(KimiChat::getChatId).list().forEach( x -> {
            Message message = new Message();
            message.setRole(x.getRole());
            message.setContent(x.getContent());
            messageList.add(message);
        });

        kimiRequest.setMessages(messageList);

        chatCompletions( kimiRequest, kimiChat.getGroupId());
        return kimiChat;
    }

    public  void upload(MultipartFile file) throws IOException, InterruptedException, ParseException {
        String boun = "----WebKitFormBoundary7MA4YWxkTrZu0gW";
        org.apache.hc.core5.http.HttpEntity httpEntity = MultipartEntityBuilder
                .create()
                .addBinaryBody("file", file.getInputStream(), ContentType.MULTIPART_FORM_DATA, file.getOriginalFilename())
                .addTextBody("purpose", "file-extract")
                .setBoundary( boun )
                .build();

        HttpPost httpPost = new HttpPost("https://api.moonshot.cn/v1/files");
        httpPost.setHeader("Authorization", "Bearer " + token.get(random.nextInt(token.size())));
//        httpPost.setHeader("Content-Type", "multipart/form-data; boundary=" + boun);
        httpPost.setEntity(httpEntity);
        CloseableHttpClient httpClient = HttpClients.createDefault();
        CloseableHttpResponse response = httpClient.execute(httpPost);
        String s = EntityUtils.toString(response.getEntity());
        log.info("上传文件 -- {}", s);


        /*Pipe pipe = Pipe.open();
        new Thread(() -> {
            try (OutputStream outputStream = Channels.newOutputStream(pipe.sink())) {
                httpEntity.writeTo(outputStream);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }).start();

        HttpRequest.BodyPublisher bodyPublisher = HttpRequest.BodyPublishers.ofInputStream(() -> Channels.newInputStream(pipe.source()));

        HttpClient client = HttpClient.newBuilder().build();

        HttpRequest request = HttpRequest.newBuilder()
                .uri(URI.create("https://api.moonshot.cn/v1/files"))
                .method("POST", bodyPublisher)
                .header(HttpHeaders.CONTENT_TYPE, "multipart/form-data;  boundary=" + boun)
                .headers("Authorization", "Bearer " + token.get(random.nextInt(token.size())))
                .build();

        HttpResponse<String> send = client.send(request, HttpResponse.BodyHandlers.ofString());
        log.info("上传文件响应 {}", send.body());*/
    }

}
