package com.sxtvs.open.api.h5template.service.impl;


import cn.hutool.core.date.DateTime;
import cn.hutool.core.io.IoUtil;
import cn.hutool.core.io.file.FileNameUtil;
import cn.hutool.http.HttpUtil;
import com.aliyun.oss.ClientException;
import com.aliyun.oss.OSS;
import com.aliyun.oss.OSSClientBuilder;
import com.aliyun.oss.OSSException;
import com.aliyun.oss.model.*;
import com.sxtvs.open.core.advice.BizException;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.util.TextUtils;
import org.apache.poi.hwpf.HWPFDocument;
import org.apache.poi.hwpf.extractor.WordExtractor;
import org.apache.poi.hwpf.usermodel.Range;
import org.apache.poi.xwpf.usermodel.XWPFDocument;
import org.apache.poi.xwpf.usermodel.XWPFParagraph;
import org.apache.poi.xwpf.usermodel.XWPFRun;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.*;
import java.net.*;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;


@Service
@Slf4j
public class OSSService {
    private final OSS ossClient = new OSSClientBuilder().build("https://oss-cn-chengdu.aliyuncs.com",
            "LTAI4GEBqfF1GX4VwsYU2Wpg",
            "rVIv0E1lRfXOCrAmkFTZnfgWiuv4ea");

    private final String bucket = "smcic-index";

    private int index = 0;

    @SneakyThrows
    public String commonPut(MultipartFile file) {
        String dir = "operate/h5Template/img";
        String extName = FileNameUtil.extName(file.getOriginalFilename());

        ByteArrayInputStream inputStream = new ByteArrayInputStream(IoUtil.readBytes(file.getInputStream()));
        String date = new DateTime().toString("yyyyMMddHHmmss");
        String fileUrl = dir+"/" + UUID.randomUUID().toString().replace("-", "") + date + "." + extName;
        ossClient.putObject(bucket, fileUrl , inputStream);
        inputStream.reset();

        return "https://cxzx.smcic.net/" + fileUrl;
    }

    @SneakyThrows
    public String urlPut(String url, String extName) {
        HttpURLConnection connection = (HttpURLConnection) new URL(url).openConnection();
        InputStream inputStream = connection.getInputStream();
        String dir = "operate/clue/img";
        String date = new DateTime().toString("yyyyMMddHHmmss");
        String fileUrl = dir+"/" + UUID.randomUUID().toString().replace("-", "") + date + "." + extName;
        ossClient.putObject(bucket, fileUrl , inputStream);
        return "https://cxzx.smcic.net/" + fileUrl;
    }

    @SneakyThrows
    public String vcrPut(MultipartFile file) {
        String date = new DateTime().toString("yyyyMMdd");
        String time = new DateTime().toString("HHmmss");
        String dir = "operate/vcr/" + date;
//        String extName = FileNameUtil.extName(file.getOriginalFilename());

        ByteArrayInputStream inputStream = new ByteArrayInputStream(IoUtil.readBytes(file.getInputStream()));
        String fileUrl = dir+"/" + time + file.getOriginalFilename();
        ossClient.putObject(bucket, fileUrl , inputStream);
        inputStream.reset();

        return "https://cxzx.smcic.net/" + fileUrl;
    }

    @SneakyThrows
    public String vcr2local(String url, String taskId) {
        HttpURLConnection connection = (HttpURLConnection) new URL(url).openConnection();
        String[] extension = url.split("/");
        InputStream inputStream = connection.getInputStream();

        String dir = "operate/vcr/thumbnails/" + taskId;
        String fileUrl = dir+"/" + extension[extension.length-1];
        ossClient.putObject(bucket, fileUrl , inputStream);
        return "https://cxzx.smcic.net/" + fileUrl;
    }

    @SneakyThrows
    public String bidPut(MultipartFile file, Long id) {

        String dir = "open/bid/" + id + "/";

        ByteArrayInputStream inputStream = new ByteArrayInputStream(IoUtil.readBytes(file.getInputStream()));
        String fileUrl = dir + file.getOriginalFilename();
        ossClient.putObject(bucket, fileUrl , inputStream);
        inputStream.reset();

        String encodedString = URLEncoder.encode(fileUrl, StandardCharsets.UTF_8);
        return "https://cxzx.smcic.net/" + encodedString;
    }

    public void bidDel(Long id){
        String prefix = "open/bid/" + id + "/";
        delete(prefix);
    }

    // 删除oss文件
    public void delete(String prefix) {

        try {
            // 删除目录及目录下的所有文件。
            String nextMarker = null;
            ObjectListing objectListing = null;
            do {
                ListObjectsRequest listObjectsRequest = new ListObjectsRequest(bucket)
                        .withPrefix(prefix)
                        .withMarker(nextMarker);

                objectListing = ossClient.listObjects(listObjectsRequest);
                if (objectListing.getObjectSummaries().size() > 0) {
                    List<String> keys = new ArrayList<String>();
                    for (OSSObjectSummary s : objectListing.getObjectSummaries()) {
                        log.info("key name: " + s.getKey());
                        keys.add(s.getKey());
                    }
                    DeleteObjectsRequest deleteObjectsRequest = new DeleteObjectsRequest(bucket).withKeys(keys).withEncodingType("url");
                    DeleteObjectsResult deleteObjectsResult = ossClient.deleteObjects(deleteObjectsRequest);
                    List<String> deletedObjects = deleteObjectsResult.getDeletedObjects();
                    for(String obj : deletedObjects) {
                        String deleteObj =  URLDecoder.decode(obj, StandardCharsets.UTF_8);
                        log.info(deleteObj);
                    }
                }

                nextMarker = objectListing.getNextMarker();
            } while (objectListing.isTruncated());
        } catch (OSSException oe) {
            log.error("Caught an OSSException, which means your request made it to OSS, "
                    + "but was rejected with an error response for some reason.");
            log.error("Error Message:" + oe.getErrorMessage());
            log.error("Error Code:" + oe.getErrorCode());
            log.error("Request ID:" + oe.getRequestId());
            log.error("Host ID:" + oe.getHostId());
        } catch (ClientException ce) {
            log.error("Caught an ClientException, which means the client encountered "
                    + "a serious internal problem while trying to communicate with OSS, "
                    + "such as not being able to access the network.");
            log.error("Error Message:" + ce.getMessage());
        }
    }



    public String bidDocPut(MultipartFile file,  Long id)  {
        try {


        // 读取DOC文件
        ByteArrayInputStream fis = new ByteArrayInputStream(IoUtil.readBytes(file.getInputStream()));

        XWPFDocument xwpfDocument = new XWPFDocument();
        HWPFDocument document = new HWPFDocument(fis);

        WordExtractor wordExtractor = new WordExtractor(document);
        String text = wordExtractor.getText();
        XWPFParagraph paragraph = xwpfDocument.createParagraph();
        XWPFRun run = paragraph.createRun();
        run.setText(text);

        ByteArrayOutputStream fos = new ByteArrayOutputStream();
        xwpfDocument.write(fos);

        String dir = "open/bid/" + id + "/";
        String fileUrl = dir + file.getOriginalFilename() + "x";
        ossClient.putObject(bucket, fileUrl , new ByteArrayInputStream(fos.toByteArray()));
        fos.close();
        fis.reset();

        String encodedString = URLEncoder.encode(fileUrl, StandardCharsets.UTF_8);
        document.close();
        xwpfDocument.close();

        return "https://cxzx.smcic.net/" + encodedString;
        }catch (IllegalArgumentException e){
            log.error("doc文件转换错误, {} {}", file.getOriginalFilename(), e.getMessage());
            return bidPut(file, id) + "x";
        }catch (Exception e){
            e.printStackTrace();
            log.error("doc文件转换错误, {} {}", file.getOriginalFilename(), e.getMessage());
            throw  new BizException("不支持的文件");
        }
    }

}


