package com.sxtvs.open.api.review.service.impl;

import com.alibaba.fastjson.JSON;
import com.sxtvs.open.api.review.dto.AuthReq;
import com.sxtvs.open.api.review.dto.AuthResp;
import com.sxtvs.open.api.review.dto.ImgReviewResp;
import com.sxtvs.open.api.review.dto.VideoReviewResp;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.*;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.zip.GZIPInputStream;

@Service
@Slf4j
public class VcrService {

    private final RestTemplate restTemplate = new RestTemplate();

    private String accessToken;

    public  void refreshToken()  {
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        headers.set("Accept-Encoding", "");
        HttpEntity<String> entity = new HttpEntity<>(JSON.toJSONString(
                new AuthReq("17691223266", "843ea558d5ef41a1877584c62762632d", "360394f5f94342639bfef63627114bb7", "b8198d5808ea4095a9e7a694f5440fd8", "59b18a90f9b6463f8fe42271d60e6405")
        ), headers);
        ResponseEntity<byte[]> response = restTemplate.postForEntity("https://home.sxtvs.net/idgate/v1/apiauth/generateCode",
                entity,
                byte[].class
        );

        byte[] respBody = response.getBody();
        if ("gzip".equals(response.getHeaders().getFirst("Content-Encoding"))){
            try {
                respBody = uncompress( respBody );
            } catch (IOException e) {
                log.info( "uncompress error:{}", e.getMessage() );
            }
        }

        AuthResp body = JSON.parseObject( new String(respBody, StandardCharsets.UTF_8), AuthResp.class );
        if (response.getStatusCode( ).is2xxSuccessful() &&  body != null && body.getResult() != null){
            this.accessToken = body.getResult().getAuthorizedCode();
            log.info("refreshToken:{}", this.accessToken);
        }else{
            log.error("获取token失败 {}", body);
        }
    }

    public ResponseEntity<ImgReviewResp> reviewImg(String json){
        HttpEntity<String> entity = new HttpEntity<>(json, getHeader());
        return restTemplate.exchange("http://10.30.160.59:80/mcr/v1/vcr/image/sync",
                HttpMethod.PUT,
                entity,
                ImgReviewResp.class);
    }

    public ResponseEntity<ImgReviewResp> getImgResult(String taskId){
        HttpEntity<String> entity = new HttpEntity<>("", getHeader());
        return restTemplate.exchange("http://10.30.160.59:80/mcr/v1/vcr/image?taskId=" + taskId,
                HttpMethod.GET,
                entity,
                ImgReviewResp.class);
    }

    private HttpHeaders getHeader(){
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        headers.set("AuthorizedCode", this.accessToken);
        return headers;
    }

    public  byte[] uncompress(byte[]  body) throws IOException {
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        try (GZIPInputStream gzipInputStream = new GZIPInputStream(new ByteArrayInputStream(body))) {
            byte[] buf = new byte[4096];
            int len = -1;
            while ((len = gzipInputStream.read(buf, 0, buf.length)) != -1) {
                byteArrayOutputStream.write(buf, 0, len);
            }
            return byteArrayOutputStream.toByteArray();
        } finally {
            byteArrayOutputStream.close();
        }

    }

    public ResponseEntity<VideoReviewResp> reviewVideo(String json){
        HttpEntity<String> entity = new HttpEntity<>(json, getHeader());
        return restTemplate.exchange("http://10.30.160.59:80/mcr/v2/vcr/media",
                HttpMethod.PUT,
                entity,
                VideoReviewResp.class);
    }

    public ResponseEntity<VideoReviewResp> getVideoResult(String taskId){
        HttpEntity<String> entity = new HttpEntity<>("", getHeader());
        return restTemplate.exchange("http://10.30.160.59:80/mcr/v2/vcr/media?taskId=" + taskId,
                HttpMethod.GET,
                entity,
                VideoReviewResp.class);
    }

}
